# deals with interacting with the freesounds.org API
from requests_oauthlib import OAuth2Session
import json
import os
import webbrowser
import requests

# define URL resources
BASE_URL = "https://freesound.org/apiv2"
SOUNDS = "/sounds"
SEARCH = "/search/text"
PACKS = "/packs"
DOWNLOAD = "/download"
AUTHORIZE = "/oauth2/authorize"
ACCESS_TOKEN = "/oauth2/access_token"
SAMPLES_PER_PAGE = 150
MIN_DOWNLOADS_TO_DOWNLOAD = 10
MIN_RATING_TO_DOWNLOAD = 4

class ApiClient:
    def __init__(self, client_key, secret_key):
        self.client_key = client_key
        self.secret_key = secret_key
        self.oauth2_code = ''
    
    # returns True if authorization was successful, False otherwise
    def oauth2_authorize(self):
        oauth = OAuth2Session(
            self.client_key, redirect_uri='https://freesound.org/home/app_permissions/permission_granted/'
        )

        authorization_url, state = oauth.authorization_url(
            BASE_URL + AUTHORIZE
        )
        # open new browser
        webbrowser.open_new(authorization_url)

        # also print URL in case of remote session
        print('please visit the following URL to achieve authorization code %s' % (authorization_url, ))

        # request code from browser
        authorization_response=input("type in authorization code: ")
        r = requests.post(
            BASE_URL + ACCESS_TOKEN,
            params={
                'client_id': self.client_key,
                'client_secret': self.secret_key,
                'grant_type': 'authorization_code',
                'code': authorization_response
            }
        )

        if r.status_code != 200:
            print('error getting access token from the given authroization code')
            return False

        access_token = r.json()['access_token']
        self.oauth2_code = access_token
        return True
    
    def filter_string(self, file_type, min_duration, max_duration, min_downloads, min_avg_rating):
        return f'type:{file_type} duration:[{min_duration} TO {max_duration}] num_downloads:[{min_downloads} TO *] is_remix:false'

    # returns json from API for processing
    def query(
            self,
            include_tags,
            exclude_tags,
            page,
            min_duration_sec,
            max_duration_sec
    ):
        results = {
            'count': 0, #total count not only this page
            'results': []
        }

        for file_type in ['wav', 'aif', 'aiff', 'mp3', 'm4a', 'flac']:
            params = {
                'token': self.secret_key,
                'query': include_tags[0],
                'filter': self.filter_string(file_type, min_duration_sec, max_duration_sec, min_downloads=MIN_DOWNLOADS_TO_DOWNLOAD, min_avg_rating=MIN_RATING_TO_DOWNLOAD),
                'page': page,
                'a_tag': 1,
                'advanced': 1,
                'page_size': SAMPLES_PER_PAGE
            }

            print(f'page: {page} file_type: {file_type} include_tags: {include_tags} exclude_tags: {exclude_tags}')
            r = requests.get(BASE_URL + SEARCH, params=params)

            if r.status_code != 200:
                print(f"error getting samples for requested query status_code: {r.status_code}")
                continue

            results_raw = r.json()
            results['count'] += results_raw['count']
            include_tags_set = set(include_tags)
            exclude_tags_set = set(exclude_tags)
            for each in results_raw['results']:
                tags = set([it.lower().strip() for it in each['tags']])
                if len(include_tags_set - tags) == 0:
                    if len(exclude_tags_set.intersection(tags)) == 0:
                        each['file_type'] = file_type
                        results['results'].append(each)

        return results

    def download_sample(self, sample_id, file_name, target_directory):
        success = True # unsuccessful will be deleted

        if not os.path.exists(target_directory + '/' + file_name):
            download_headers = {'Authorization': 'Bearer ' + self.oauth2_code}
            url = BASE_URL + SOUNDS + '/' + sample_id + DOWNLOAD
            response = requests.get(url, headers=download_headers)
            if response.status_code != 200:
                print('error downloading file with sample id %s' % (sample_id, ))
                success = False
            else:
                with open(target_directory + '/' + file_name, 'wb') as f:
                    f.write(response.content)

        return success

