# main driver for sounds scraper
from client import ApiClient
from data import SampleData
from tqdm import tqdm
import argparse
import csv
import os
import json

# used to split of CSV files to run separately and avoid rate limit issues when downloading
from file_utils import FileUtils

SPLIT_LENGTH = 1900

# attempts to retrieve a number of samples given a specific query and store them into data array
def retrieve_and_process(
        client,
        data,
        include_tags,
        exclude_tags,
        num_samples,
        min_duration_sec,
        max_duration_sec
):
    print(f"running query include_tags: {include_tags}, exclude_tags: {exclude_tags} num_samples: {num_samples}")

    response = client.query(
        include_tags,
        exclude_tags,
        page=1,
        min_duration_sec=min_duration_sec,
        max_duration_sec=max_duration_sec
    )

    # can only retrieve at most number of samples available on website
    samples_to_retrieve = response['count']
    if num_samples > 0:
        samples_to_retrieve = min(response['count'], num_samples)
    print(f'samples_to_retrieve: {samples_to_retrieve}')

    if samples_to_retrieve == 0:
            print("no samples found!")
            return

    samples_to_retrieve -= data.process_samples(response['results'], samples_to_retrieve)
    
    page = 2
    while samples_to_retrieve > 0:
        response = client.query(
            include_tags,
            exclude_tags,
            page,
            min_duration_sec=min_duration_sec,
            max_duration_sec=max_duration_sec
        )
        page += 1
        if response == {} or response['count'] == 0:
            print("no more samples found!")
            break
        samples_to_retrieve -= data.process_samples(response['results'], samples_to_retrieve)


def retrieve_samples(args):

    path_query_csv = f'./input/query.csv'

    if not os.path.isfile(path_query_csv):
        print("could not find file %s" % (path_query_csv, ))
        return None

    # load credentials JSON file
    with open('./input/credentials.json') as f:
        data = json.load(f)
        CLIENT_ID = data['client_id']
        SECRET_KEY = data['client_secret']

    # define classes for API for data management
    client = ApiClient(CLIENT_ID, SECRET_KEY)
    data = SampleData()

    def extract_tags(col):
        col_strip: str = col.strip()
        return col_strip.split(';')

    with open(path_query_csv, 'r') as csvfile:
        reader = csv.reader(csvfile, delimiter=',')
        for row in reader:
            include_tags = extract_tags(row[0])
            exclude_tags = extract_tags(row[1])
            retrieve_and_process(
                client,
                data,
                include_tags,
                exclude_tags,
                num_samples=int(row[-1]),
                min_duration_sec=args.min_duration_sec,
                max_duration_sec=args.max_duration_sec
            )
    
    return data

def download(data, download_location):
    # load credentials JSON file
    with open('./input/credentials.json') as f:
        file_data = json.load(f)
        CLIENT_ID = file_data['client_id']
        SECRET_KEY = file_data['client_secret']

    # need oauth2 to download
    client = ApiClient(CLIENT_ID, SECRET_KEY)
    client.oauth2_authorize()
    data_to_download = data.data_array

    print("downloading %d samples to %s" % (data_to_download.shape[0], download_location))

    unsuccessful_downloads = []

    for row in tqdm(data_to_download):
        success = client.download_sample(sample_id=row[0], file_name=row[1], target_directory=download_location)
        if not success:
            unsuccessful_downloads.append(row)
    
    # delete unsuccessful downloads from data array
    data.remove_unsuccessful(unsuccessful_downloads)


def main(args):
    if args.path_download_csv:
        download_location = './output/audio'
        if not os.path.isdir(download_location):
            os.mkdir(download_location)
        download_csv(args.path_download_csv, download_location)
    else:
        data: SampleData = retrieve_samples(args)

        if data.data_array.shape[0] > 0:
            data.remove_duplicates()

        for file in FileUtils.listSubFiles('./output/csv'):
            loaded_data = SampleData()
            loaded_data.load_from_csv(file)
            data.combine(loaded_data)
            data.remove_duplicates()

        if not os.path.isdir('./output/csv'):
            os.mkdir('./output/csv')
        data.save_to_csv_split('./output/csv/freesound', SPLIT_LENGTH)


def download_csv(csv_to_download, download_location):
    data = SampleData()
    data.load_from_csv(csv_to_download)
    if not os.path.isdir(download_location):
        print("folder %s not found, creating" % (download_location, ))
        os.mkdir(download_location)
    download(data, download_location)
    data.save_to_csv(csv_to_download + '-downloaded')


if __name__== "__main__":
    # setup argument parser
    parser = argparse.ArgumentParser()

    parser.add_argument('-path_download_csv', type=str)

    parser.add_argument('-min_duration_sec', type=float, default=5.0)
    parser.add_argument('-max_duration_sec', type=float, default=30.0)
    args = parser.parse_args()

    # call main function
    main(args)